package sun.swing.text;

import javax.swing.text.BadLocationException;
import java.awt.Dimension;
import javax.swing.JViewport;
import java.lang.reflect.InvocationTargetException;
import javax.swing.text.AbstractDocument;
import java.awt.Insets;
import java.awt.ComponentOrientation;
import java.awt.Rectangle;
import javax.swing.border.TitledBorder;
import javax.swing.BorderFactory;
import java.awt.print.PrinterException;
import java.awt.Graphics2D;
import java.awt.print.PageFormat;
import java.awt.Graphics;
import javax.swing.text.EditorKit;
import sun.font.FontDesignMetrics;
import java.awt.FontMetrics;
import javax.swing.JTextPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JPasswordField;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.FutureTask;
import java.util.concurrent.Callable;
import java.util.Collections;
import javax.swing.SwingUtilities;
import javax.swing.CellRendererPane;
import java.awt.Component;
import sun.swing.text.html.FrameEditorPaneTag;
import java.awt.Container;
import javax.swing.text.html.HTML;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.Document;
import java.util.Iterator;
import java.util.ArrayList;
import javax.swing.JEditorPane;
import java.awt.print.Printable;
import java.util.List;
import java.awt.Font;
import java.text.MessageFormat;
import java.awt.font.FontRenderContext;
import java.util.concurrent.atomic.AtomicReference;
import javax.swing.text.JTextComponent;

public class TextComponentPrintable implements CountingPrintable
{
    private static final int LIST_SIZE = 1000;
    private boolean isLayouted;
    private final JTextComponent textComponentToPrint;
    private final AtomicReference<FontRenderContext> frc;
    private final JTextComponent printShell;
    private final MessageFormat headerFormat;
    private final MessageFormat footerFormat;
    private static final float HEADER_FONT_SIZE = 18.0f;
    private static final float FOOTER_FONT_SIZE = 12.0f;
    private final Font headerFont;
    private final Font footerFont;
    private final List<TextComponentPrintable.IntegerSegment> rowsMetrics;
    private final List<TextComponentPrintable.IntegerSegment> pagesMetrics;
    private boolean needReadLock;
    
    public static Printable getPrintable(JTextComponent textComponent, MessageFormat messageFormat, MessageFormat messageFormat2) {
        if (textComponent instanceof JEditorPane && isFrameSetDocument(textComponent.getDocument())) {
            List<JEditorPane> frames = getFrames((JEditorPane)textComponent);
            ArrayList<Printable> list = new ArrayList();
            Iterator<Object> iterator = frames.iterator();
            while (iterator.hasNext()) {
                list.add((Printable)getPrintable((JEditorPane)iterator.next(), messageFormat, messageFormat2));
            }
            return new CompoundPrintable(list);
        }
        return new TextComponentPrintable(textComponent, messageFormat, messageFormat2);
    }
    
    private static boolean isFrameSetDocument(Document document) {
        boolean n = false;
        if (document instanceof HTMLDocument && ((HTMLDocument)document).getIterator(HTML.Tag.FRAME).isValid()) {
            n = true;
        }
        return n;
    }
    
    private static List<JEditorPane> getFrames(JEditorPane editorPane) {
        ArrayList<JEditorPane> list = new ArrayList();
        getFrames(editorPane, list);
        if (list.size() == 0) {
            createFrames(editorPane);
            getFrames(editorPane, list);
            return list;
        }
        return list;
    }
    
    private static void getFrames(Container container, List<JEditorPane> list) {
        for (Component component : container.getComponents()) {
            if (component instanceof FrameEditorPaneTag && component instanceof JEditorPane) {
                list.add((JEditorPane)component);
            }
            else if ((component instanceof Container)) {
                getFrames((Container)component, list);
            }
        }
    }
    
    private static void createFrames(final JEditorPane editorPane) {
        Runnable doRun = new Runnable() {
            @Override
            public void run() {
                CellRendererPane cellRendererPane = new CellRendererPane();
                cellRendererPane.add(editorPane);
                cellRendererPane.setSize(500, 500);
            }
        };
        if (SwingUtilities.isEventDispatchThread()) {
            doRun.run();
        }
        else {
            try {
                SwingUtilities.invokeAndWait(doRun);
            }
            catch (Exception cause) {
                if ((cause instanceof RuntimeException)) {
                    throw (RuntimeException)cause;
                }
                throw new RuntimeException(cause);
            }
        }
    }
    
    private TextComponentPrintable(JTextComponent textComponentToPrint, MessageFormat headerFormat, MessageFormat footerFormat) {
        this.isLayouted = false;
        this.frc = new AtomicReference(null);
        this.needReadLock = false;
        this.textComponentToPrint = textComponentToPrint;
        this.headerFormat = headerFormat;
        this.footerFormat = footerFormat;
        this.headerFont = textComponentToPrint.getFont().deriveFont(1, 18.0f);
        this.footerFont = textComponentToPrint.getFont().deriveFont(0, 12.0f);
        this.pagesMetrics = Collections.synchronizedList(new ArrayList());
        this.rowsMetrics = new ArrayList(1000);
        this.printShell = this.createPrintShell(textComponentToPrint);
    }
    
    private JTextComponent createPrintShell(final JTextComponent textComponent) {
        if (SwingUtilities.isEventDispatchThread()) {
            return this.createPrintShellOnEDT(textComponent);
        }
        FutureTask<Object> doRun = new FutureTask(new Callable<JTextComponent>() {
            @Override
            public JTextComponent call() throws Exception {
                return TextComponentPrintable.access$000(TextComponentPrintable.this, textComponent);
            }
        });
        SwingUtilities.invokeLater(doRun);
        try {
            return (JTextComponent)doRun.get();
        }
        catch (InterruptedException cause) {
            throw new RuntimeException(cause);
        }
        catch (ExecutionException ex) {
            Throwable cause2 = ex.getCause();
            if ((cause2 instanceof Error)) {
                throw (Error)cause2;
            }
            if ((cause2 instanceof RuntimeException)) {
                throw (RuntimeException)cause2;
            }
            throw new AssertionError(cause2);
        }
    }
    
    private JTextComponent createPrintShellOnEDT(final JTextComponent textComponent) {
        assert !(!SwingUtilities.isEventDispatchThread());
        JTextComponent textComponent2 = null;
        if ((textComponent instanceof JPasswordField)) {
            textComponent2 = new JPasswordField() {
                {
                    this.setEchoChar(((JPasswordField)textComponent).getEchoChar());
                    this.setHorizontalAlignment(((JTextField)textComponent).getHorizontalAlignment());
                }
                
                @Override
                public FontMetrics getFontMetrics(Font font) {
                    return (TextComponentPrintable.this.frc.get() == null) ? super.getFontMetrics(font) : FontDesignMetrics.getMetrics(font, (FontRenderContext)TextComponentPrintable.this.frc.get());
                }
            };
        }
        else if ((textComponent instanceof JTextField)) {
            textComponent2 = new JTextField() {
                {
                    this.setHorizontalAlignment(((JTextField)textComponent).getHorizontalAlignment());
                }
                
                @Override
                public FontMetrics getFontMetrics(Font font) {
                    return (TextComponentPrintable.this.frc.get() == null) ? super.getFontMetrics(font) : FontDesignMetrics.getMetrics(font, (FontRenderContext)TextComponentPrintable.this.frc.get());
                }
            };
        }
        else if ((textComponent instanceof JTextArea)) {
            textComponent2 = new JTextArea() {
                {
                    JTextArea textArea = (JTextArea)textComponent;
                    this.setLineWrap(textArea.getLineWrap());
                    this.setWrapStyleWord(textArea.getWrapStyleWord());
                    this.setTabSize(textArea.getTabSize());
                }
                
                @Override
                public FontMetrics getFontMetrics(Font font) {
                    return (TextComponentPrintable.this.frc.get() == null) ? super.getFontMetrics(font) : FontDesignMetrics.getMetrics(font, (FontRenderContext)TextComponentPrintable.this.frc.get());
                }
            };
        }
        else if ((textComponent instanceof JTextPane)) {
            textComponent2 = new JTextPane() {
                @Override
                public FontMetrics getFontMetrics(Font font) {
                    return (TextComponentPrintable.this.frc.get() == null) ? super.getFontMetrics(font) : FontDesignMetrics.getMetrics(font, (FontRenderContext)TextComponentPrintable.this.frc.get());
                }
                
                @Override
                public EditorKit getEditorKit() {
                    if (this.getDocument() == textComponent.getDocument()) {
                        return ((JTextPane)textComponent).getEditorKit();
                    }
                    return super.getEditorKit();
                }
            };
        }
        else if ((textComponent instanceof JEditorPane)) {
            textComponent2 = new JEditorPane() {
                @Override
                public FontMetrics getFontMetrics(Font font) {
                    return (TextComponentPrintable.this.frc.get() == null) ? super.getFontMetrics(font) : FontDesignMetrics.getMetrics(font, (FontRenderContext)TextComponentPrintable.this.frc.get());
                }
                
                @Override
                public EditorKit getEditorKit() {
                    if (this.getDocument() == textComponent.getDocument()) {
                        return ((JEditorPane)textComponent).getEditorKit();
                    }
                    return super.getEditorKit();
                }
            };
        }
        textComponent2.setBorder(null);
        textComponent2.setOpaque(textComponent.isOpaque());
        textComponent2.setEditable(textComponent.isEditable());
        textComponent2.setEnabled(textComponent.isEnabled());
        textComponent2.setFont(textComponent.getFont());
        textComponent2.setBackground(textComponent.getBackground());
        textComponent2.setForeground(textComponent.getForeground());
        textComponent2.setComponentOrientation(textComponent.getComponentOrientation());
        if ((textComponent2 instanceof JEditorPane)) {
            textComponent2.putClientProperty("JEditorPane.honorDisplayProperties", textComponent.getClientProperty("JEditorPane.honorDisplayProperties"));
            textComponent2.putClientProperty("JEditorPane.w3cLengthUnits", textComponent.getClientProperty("JEditorPane.w3cLengthUnits"));
            textComponent2.putClientProperty("charset", textComponent.getClientProperty("charset"));
        }
        textComponent2.setDocument(textComponent.getDocument());
        return textComponent2;
    }
    
    @Override
    public int getNumberOfPages() {
        return this.pagesMetrics.size();
    }
    
    @Override
    public int print(final Graphics graphics, final PageFormat pageFormat, final int n) throws PrinterException {
        if (!this.isLayouted) {
            if ((graphics instanceof Graphics2D)) {
                this.frc.set(((Graphics2D)graphics).getFontRenderContext());
            }
            this.layout((int)Math.floor(pageFormat.getImageableWidth()));
            this.calculateRowsMetrics();
        }
        int n2;
        if (!SwingUtilities.isEventDispatchThread()) {
            FutureTask<Object> doRun = new FutureTask(new Callable<Integer>() {
                @Override
                public Integer call() throws Exception {
                    return Integer.valueOf(TextComponentPrintable.access$200(TextComponentPrintable.this, graphics, pageFormat, n));
                }
            });
            SwingUtilities.invokeLater(doRun);
            try {
                n2 = (int)doRun.get();
            }
            catch (InterruptedException cause) {
                throw new RuntimeException(cause);
            }
            catch (ExecutionException ex) {
                Throwable cause2 = ex.getCause();
                if ((cause2 instanceof PrinterException)) {
                    throw (PrinterException)cause2;
                }
                if ((cause2 instanceof RuntimeException)) {
                    throw (RuntimeException)cause2;
                }
                if ((cause2 instanceof Error)) {
                    throw (Error)cause2;
                }
                throw new RuntimeException(cause2);
            }
        }
        else {
            n2 = this.printOnEDT(graphics, pageFormat, n);
        }
        return n2;
    }
    
    private int printOnEDT(Graphics graphics, PageFormat pageFormat, int n) throws PrinterException {
        assert !(!SwingUtilities.isEventDispatchThread());
        Object emptyBorder = BorderFactory.createEmptyBorder();
        if (this.headerFormat != null || this.footerFormat != null) {
            Object[] array = { Integer.valueOf(n + 1) };
            if (this.headerFormat != null) {
                emptyBorder = new TitledBorder(emptyBorder, this.headerFormat.format(array), 2, 1, this.headerFont, this.printShell.getForeground());
            }
            if (this.footerFormat != null) {
                emptyBorder = new TitledBorder(emptyBorder, this.footerFormat.format(array), 2, 6, this.footerFont, this.printShell.getForeground());
            }
        }
        Insets borderInsets = emptyBorder.getBorderInsets(this.printShell);
        this.updatePagesMetrics(n, (int)Math.floor(pageFormat.getImageableHeight()) - borderInsets.top - borderInsets.bottom);
        if (this.pagesMetrics.size() <= n) {
            return 1;
        }
        Graphics2D g = (Graphics2D)graphics.create();
        g.translate(pageFormat.getImageableX(), pageFormat.getImageableY());
        emptyBorder.paintBorder(this.printShell, g, 0, 0, (int)Math.floor(pageFormat.getImageableWidth()), (int)Math.floor(pageFormat.getImageableHeight()));
        g.translate(0, borderInsets.top);
        g.clip(new Rectangle(0, 0, (int)pageFormat.getWidth(), ((TextComponentPrintable.IntegerSegment)this.pagesMetrics.get(n)).end - ((TextComponentPrintable.IntegerSegment)this.pagesMetrics.get(n)).start + 1));
        int n2 = 0;
        if (ComponentOrientation.RIGHT_TO_LEFT == this.printShell.getComponentOrientation()) {
            n2 = (int)pageFormat.getImageableWidth() - this.printShell.getWidth();
        }
        g.translate(n2, -((TextComponentPrintable.IntegerSegment)this.pagesMetrics.get(n)).start);
        this.printShell.print(g);
        g.dispose();
        return 0;
    }
    
    private void releaseReadLock() {
        assert !SwingUtilities.isEventDispatchThread();
        Document document = this.textComponentToPrint.getDocument();
        if ((document instanceof AbstractDocument)) {
            try {
                ((AbstractDocument)document).readUnlock();
                this.needReadLock = true;
            }
            catch (Error error) {}
        }
        return;
    }
    
    private void acquireReadLock() {
        assert !SwingUtilities.isEventDispatchThread();
        if (this.needReadLock) {
            try {
                SwingUtilities.invokeAndWait(new Runnable() {
                    @Override
                    public void run() {
                    }
                });
            }
            catch (InterruptedException ex) {}
            catch (InvocationTargetException ex2) {}
            ((AbstractDocument)this.textComponentToPrint.getDocument()).readLock();
            this.needReadLock = false;
        }
        return;
    }
    
    private void layout(final int n) {
        if (!SwingUtilities.isEventDispatchThread()) {
            FutureTask<Object> doRun = new FutureTask(new Callable<Object>() {
                @Override
                public Object call() throws Exception {
                    TextComponentPrintable.access$300(TextComponentPrintable.this, n);
                    return null;
                }
            });
            this.releaseReadLock();
            SwingUtilities.invokeLater(doRun);
            try {
                doRun.get();
            }
            catch (InterruptedException cause) {
                throw new RuntimeException(cause);
            }
            catch (ExecutionException ex) {
                Throwable cause2 = ex.getCause();
                if ((cause2 instanceof RuntimeException)) {
                    throw (RuntimeException)cause2;
                }
                if ((cause2 instanceof Error)) {
                    throw (Error)cause2;
                }
                throw new RuntimeException(cause2);
            }
            finally {
                this.acquireReadLock();
            }
        }
        else {
            this.layoutOnEDT(n);
        }
        this.isLayouted = true;
    }
    
    private void layoutOnEDT(int width) {
        assert !(!SwingUtilities.isEventDispatchThread());
        CellRendererPane cellRendererPane = new CellRendererPane();
        JViewport comp = new JViewport();
        comp.setBorder(null);
        Dimension dimension = new Dimension(width, 2147482647);
        if ((this.printShell instanceof JTextField)) {
            dimension = new Dimension(dimension.width, this.printShell.getPreferredSize().height);
        }
        this.printShell.setSize(dimension);
        comp.setComponentOrientation(this.printShell.getComponentOrientation());
        comp.setSize(dimension);
        comp.add(this.printShell);
        cellRendererPane.add(comp);
        return;
    }
    
    private void updatePagesMetrics(int n, int n2) {
        while (n >= this.pagesMetrics.size() && !this.rowsMetrics.isEmpty()) {
            int n3 = this.pagesMetrics.size() - 1;
            int n4;
            int n5;
            for (n4 = ((n3 >= 0) ? (((TextComponentPrintable.IntegerSegment)this.pagesMetrics.get(n3)).end + 1) : 0), n5 = 0; n5 < this.rowsMetrics.size() && ((TextComponentPrintable.IntegerSegment)this.rowsMetrics.get(n5)).end - n4 + 1 <= n2; ++n5) {}
            if (n5 == 0) {
                this.pagesMetrics.add(new TextComponentPrintable.IntegerSegment(n4, n4 + n2 - 1));
            }
            else {
                --n5;
                this.pagesMetrics.add(new TextComponentPrintable.IntegerSegment(n4, ((TextComponentPrintable.IntegerSegment)this.rowsMetrics.get(n5)).end));
                for (int i = 0; i <= n5; ++i) {
                    this.rowsMetrics.remove(0);
                }
            }
        }
    }
    
    private void calculateRowsMetrics() {
        int length = this.printShell.getDocument().getLength();
        ArrayList list = new ArrayList(1000);
        int i = 0;
        int n = -1;
        int n2 = -1;
        while (i < length) {
            try {
                Rectangle modelToView = this.printShell.modelToView(i);
                if (modelToView != null) {
                    int n3 = (int)modelToView.getY();
                    int n4 = (int)modelToView.getHeight();
                    if (n4 != 0 && (n3 != n || n4 != n2)) {
                        n = n3;
                        n2 = n4;
                        list.add(new TextComponentPrintable.IntegerSegment(n3, n3 + n4 - 1));
                    }
                }
            }
            catch (BadLocationException ex) {
                assert false;
            }
            ++i;
        }
        Collections.sort(list);
        int start = Integer.MIN_VALUE;
        int n5 = Integer.MIN_VALUE;
        for (TextComponentPrintable.IntegerSegment integerSegment : list) {
            if (n5 < integerSegment.start) {
                if (n5 != Integer.MIN_VALUE) {
                    this.rowsMetrics.add(new IntegerSegment(start, n5));
                }
                start = integerSegment.start;
                n5 = integerSegment.end;
            }
            else {
                n5 = integerSegment.end;
            }
        }
        if (n5 != Integer.MIN_VALUE) {
            this.rowsMetrics.add(new IntegerSegment(start, n5));
        }
    }
    
    static JTextComponent access$000(TextComponentPrintable textComponentPrintable, JTextComponent textComponent) {
        return textComponentPrintable.createPrintShellOnEDT(textComponent);
    }
    
    static int access$200(TextComponentPrintable textComponentPrintable, Graphics graphics, PageFormat pageFormat, int n) throws PrinterException {
        return textComponentPrintable.printOnEDT(graphics, pageFormat, n);
    }
    
    static void access$300(TextComponentPrintable textComponentPrintable, int n) {
        textComponentPrintable.layoutOnEDT(n);
    }
    
    private static class IntegerSegment implements Comparable<TextComponentPrintable.IntegerSegment>
    {
        final int start;
        final int end;
        
        IntegerSegment(int start, int end) {
            this.start = start;
            this.end = end;
        }
        
        @Override
        public int compareTo(TextComponentPrintable.IntegerSegment integerSegment) {
            int n = this.start - integerSegment.start;
            return (n != 0) ? n : (this.end - integerSegment.end);
        }
        
        @Override
        public boolean equals(Object o) {
            return o instanceof TextComponentPrintable.IntegerSegment && this.compareTo((TextComponentPrintable.IntegerSegment)o) == 0;
        }
        
        @Override
        public int hashCode() {
            return 37 * (37 * 17 + this.start) + this.end;
        }
        
        @Override
        public String toString() {
            return "IntegerSegment [" + this.start + ", " + this.end + "]";
        }
    }
}
